<?php
declare(strict_types=1);

namespace Intergo\LaravelAuthMiddleware\Http\Middleware\Web;

use Auth;
use Closure;
use Illuminate\Http\JsonResponse;

use function App\Http\Middleware\response;

class CheckPermissionWeb
{
    /**
     * Handle the incoming request.
     *
     * @param $request
     * @param Closure $next
     * @param string $permission
     * @return JsonResponse|mixed
     */
    public function handle($request, Closure $next, string $permission)
    {
        $user = Auth::user();
        $permissions = explode('|', $permission);
        if ($user->hasAnyPermission($permissions)) {
            return $next($request);
        }

        return response()->json([
            'success' => false,
            'status' => 403,
            'message' => "Access Denied. You do not have permission to {$permission}."
        ], 403);
    }
}
